// ignore: implementation_imports
import 'package:intl/src/intl_helpers.dart';
import 'package:lokalise_flutter_sdk/ota/data/sdk_ota_data.dart';
import 'package:lokalise_flutter_sdk/ota/exceptions/lokalise_exception.dart';
import 'package:lokalise_flutter_sdk/ota/model/translation_result.dart';
import 'package:lokalise_flutter_sdk/ota/service/bundle_service.dart';
import 'package:lokalise_flutter_sdk/ota/service/proxy.dart';
import 'package:lokalise_flutter_sdk/ota/service/sdk_store.dart';
import 'package:package_info_plus/package_info_plus.dart';

class Lokalise {
  static Lokalise? _instance;

  final String _sdkToken;
  final String _projectId;
  final bool _preRelease;
  late String _appVersion;

  late Future<void> _loadDataFuture;

  Lokalise._internal(
      String projectId, String sdkToken, String? appVersion, bool? preRelease)
      : _projectId = projectId,
        _sdkToken = sdkToken,
        _appVersion = appVersion ?? '',
        _preRelease = preRelease ?? false {
    messageLookup = MessageLookupProxy.from(messageLookup);
    _loadDataFuture = _loadCachedData();
  }

  static Future<void> init(
      {required String projectId,
      required String sdkToken,
      String? appVersion,
      bool? preRelease}) {
    assert(projectId.isNotEmpty, 'Lokalise project ID could not be empty');
    assert(sdkToken.isNotEmpty, 'Lokalise token could not be empty');

    _instance = Lokalise._internal(projectId, sdkToken, appVersion, preRelease);
    return _instance!._loadDataFuture;
  }

  static Lokalise get instance {
    assert(Lokalise._instance != null,
        'Lokalise must be initialized before using it');
    return Lokalise._instance!;
  }

  Future<void> _loadCachedData() async {
    BundleService.init(projectId: _projectId, sdkToken: _sdkToken);
    await SdkStore.init();
    SdkOtaData.releaseData =
        await BundleService.instance.getStoredReleaseData(_projectId);

    if (_appVersion == '') {
      final info = await PackageInfo.fromPlatform();
      _appVersion = info.version;
    }
  }

  Future<UpdateResult> update() async {
    await _loadDataFuture;

    int? transVersion = SdkOtaData.releaseVersion;
    transVersion ??= 0;

    try {
      int newTransVersion = await BundleService.instance.getBundle(
        preRelease: _preRelease,
        transVersion: transVersion,
        appVersion: _appVersion,
      );
      return UpdateResult(transVersion, newTransVersion);
    } on LokaliseException {
      rethrow;
    } catch (e) {
      throw LokaliseException('Unexpected exception');
    }
  }

  /// Save cache to local storage (all languages)
  set metadata(Map<String, List<String>> otadata) {
    SdkOtaData.otadata = otadata;
  }

  /// Check if current device has data cache
  bool get hasMetadata {
    return SdkOtaData.otadata != null;
  }
}
