import 'package:lokalise_flutter_sdk/ota/model/release_data.dart';
import 'package:lokalise_flutter_sdk/ota/service/logger.dart';
import 'package:lokalise_flutter_sdk/otav2/data/persistence/bundle_persistence.dart';
import 'package:lokalise_flutter_sdk/otav2/data/repositories/local_bundle_repository.dart';
import 'package:lokalise_flutter_sdk/otav2/domain/models/bundle.dart';
import 'package:shared_preferences/shared_preferences.dart';

class SdkStore {
  static final _log = getLogger();

  static late SdkStore _instance;

  /// Temporal solution to be able to inject all the needed dependencies
  /// by the repo layer without have to change too much on the legacy code
  static Future<void> init() async {
    _instance = SdkStore(
      localRepo: LocalBundleRepository(
        persistence: BundlePersistence(
          sharedPreferences: await SharedPreferences.getInstance(),
        ),
      ),
    );
  }

  static SdkStore get instance => _instance;

  final LocalBundleRepository _localRepo;

  SdkStore({required LocalBundleRepository localRepo}) : _localRepo = localRepo;

  Future<void> removeData() async {
    try {
      await _localRepo.removeBundle();
    } catch (e) {
      _log.w('Failed to clear cached translations.', e);
    }
  }

  Future<void> save(Bundle bundle) async {
    try {
      await _localRepo.saveBundle(bundle: bundle);
    } catch (e) {
      _log.w('Failed to cache translations.', e);
    }
  }

  Future<ReleaseData?> getReleaseData(String projectId) async {
    ReleaseData? res;
    try {
      final bundle = _localRepo.getBundle();
      if (bundle == null) return null;
      if (bundle.projectId != projectId) {
        await removeData();
        return null;
      }
      res = ReleaseData(
        version: bundle.translationVersion,
        // ignore: deprecated_member_use_from_same_package
        data: bundle.legacyTranslations,
      );
    } catch (e) {
      _log.w('Failed to load cached translations.', e);
    }

    return res;
  }
}
