import 'dart:convert';
import 'package:http/http.dart';
import 'package:lokalise_flutter_sdk/otav2/data/api/clients/ota/response/error/ota_error_response.dart';
import 'package:lokalise_flutter_sdk/otav2/data/api/clients/ota/response/ota_response.dart';
import 'package:lokalise_flutter_sdk/otav2/data/api/clients/ota/response/success/get_bundle_response.dart';
import 'package:lokalise_flutter_sdk/otav2/data/api/exceptions/api_exception.dart';
import 'package:lokalise_flutter_sdk/otav2/data/api/requests/bundle_request.dart';
import 'package:lokalise_flutter_sdk/otav2/domain/models/credentials.dart';

class OtaClient extends BaseClient {
  static const String _domain = 'ota.lokalise.com';

  final Client _inner;
  OtaClient({required Client httpClient}) : _inner = httpClient;

  @override
  Future<StreamedResponse> send(BaseRequest request) {
    request.headers['content-type'] = 'application/json';

    return _inner.send(request);
  }

  Future<OtaResponse<GetBundleResponse>> getBundle({
    required Credentials credentials,
    required BundleRequest request,
  }) async {
    final url = Uri.https(
      _domain,
      'v3/lokalise/projects/${credentials.projectId}/frameworks/flutter_sdk',
      request.toQueryParams(),
    );
    final response = await this.get(
      url,
      headers: {'x-ota-api-token': credentials.token},
    );

    final code = response.statusCode;
    if (code == 200) {
      return OtaResponse.success(
        httpCode: code,
        result: GetBundleResponse.fromJson(json: jsonDecode(response.body)),
      );
    } else if (code == 204) {
      return OtaResponse.success(
        httpCode: code,
        result: GetBundleResponse.empty(),
      );
    } else if (code >= 400 && code < 500) {
      return OtaResponse.failure(
        httpCode: code,
        error: OtaErrorResponse.fromJson(json: jsonDecode(response.body)),
      );
    } else {
      throw ApiException(
        httpCode: code,
        responseBody: response.body,
      );
    }
  }
}
